# -*- coding: utf-8 -*-

"""
CLI entry point
"""
# enable absolute imports of this module for Python2.x
from __future__ import absolute_import

from .cli import main

if __name__ == '__main__':
    main()
